/*
    Java class with two subclasses
    We would take Employee as parent class
    Subclasses are:
    1. FullTimeEmployee
    2. HourlyEmployee
 */

abstract class Employee {

    // Required attributes

    protected String name;
    protected int age;
    protected String address;
    protected String phone;

    // Constructor

    public Employee(String name, int age, String address, String phone) {
        this.name = name;
        this.age = age;
        this.address = address;
        this.phone = phone;
    }

    // Setters & Getters

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public int getAge() {
        return age;
    }

    public void setAge(int age) {
        this.age = age;
    }

    public String getAddress() {
        return address;
    }

    public void setAddress(String address) {
        this.address = address;
    }

    public String getPhone() {
        return phone;
    }

    public void setPhone(String phone) {
        this.phone = phone;
    }

    // A toString method which returns string representation of Employee

    public String toString() {
        return "Name: " + name + "\nAge = " + age + "\nAddress: " + address + "\nPhone Number: " + phone;
    }

    // An abstract method which would be implemented by child classes which returns salary of employee

    abstract double calculateSalary();
}

// Child classes

// FullTimeEmployee class

class FullTimeEmployee extends  Employee {

    // A unique attribute of FullTimeEmployee

    private double annualSalary;

    // Constructor

    public FullTimeEmployee(String name, int age, String address, String phone, double annualSalary) {
        super(name, age, address, phone);
        this.annualSalary = annualSalary;
    }

    // Setter & Getter

    public double getAnnualSalary() {
        return annualSalary;
    }

    public void setAnnualSalary(double annualSalary) {
        this.annualSalary = annualSalary;
    }

    // Implementing calculateSalary method for this class

    @Override
    double calculateSalary() {
        return annualSalary;
    }
}

// HourlyEmployee class

class HourlyEmployee extends Employee {

    // Unique attributes which has hourly salary & number of hours worked

    private int numHours;
    private double hourlySalary;

    // Constructor, Setter & Getter

    public HourlyEmployee(String name, int age, String address, String phone, int numHours, double hourlySalary) {
        super(name, age, address, phone);
        this.numHours = numHours;
        this.hourlySalary = hourlySalary;
    }

    public int getNumHours() {
        return numHours;
    }

    public void setNumHours(int numHours) {
        this.numHours = numHours;
    }

    public double getHourlySalary() {
        return hourlySalary;
    }

    public void setHourlySalary(double hourlySalary) {
        this.hourlySalary = hourlySalary;
    }

    // Implementing calculateSalary method

    @Override
    double calculateSalary() {
        return hourlySalary * numHours;
    }
}

public class Main {

    // Testing classes

    public static void main(String[] args) {

        Employee emp1 = new FullTimeEmployee("Sarah", 20, "Washington DC", "12345", 800);

        Employee emp2 = new HourlyEmployee("John", 25, "Seattle", "24795", 10, 50);

        System.out.println("*** Full Time Employee ***");
        System.out.println(emp1);
        System.out.println("Salary: " + emp1.calculateSalary());

        System.out.println();

        System.out.println("*** Hourly Employee ***");
        System.out.println(emp2);
        System.out.println("Salary: " + emp2.calculateSalary());
    }
}
